NAME
        FioCheckFileClear - Check if a file exists

SYNOPSIS

        #include <stdio.h>
        #include <errno.h>
        #include <sys/types.h>
        #include <sys/stat.h>
        #include <sys/timeb.h>
        #include <fio.h>

        SHORT APIENTRY
        MemHepDeque(psHeap,psPriority,ppvData)
        HEAP_P psHeap;
        PSHORT psPriority;
        PVOID * ppvData;

DESCRIPTION

        The priority and pvData from psHeap -> psHeapData[0] are
        returned.  The heap is adjusted as necessary after the
        call.

        HEAP_P  psHeap;    /* main heap structure */
        PVOID   pvData;    /* for MemHepDeq() */
        SHORT   sPriority; /* for MemHepDeq() */

        C_STATUS = MemHepInit(&psHeap,10);  /* init for 10 members */
        
        C_STATUS = MemHepEnq(psHeap, 7, ptrToData);  /* prio = 7 */

        C_STATUS = MemHepDeq(psHeap, &sPriority, &pvData);


RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      = element returned
        C_NOTOK = heap empty


NAME
        MemHepDown - adjusts a heap after a deque

SYNOPSIS

        #include <memincs.h>

        SHORT APIENTRY
        MemHepDown(psHeap, sRoot)
        HEAP_P psHeap;
        SHORT sRoot;

DESCRIPTION

        Adjusts heap psHeap starting from sRoot.  This routine is
        called by MemHepDeque() and there is not reason to call it
        in your application.
      
RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      - always returned


NAME
        MemHepEnque - adds an element and its priority to a heap

SYNOPSIS

        #include <memincs.h>

        SHORT APIENTRY
        MemHepEnque(psHeap,sPriority,pvData)
        HEAP_P psHeap;
        SHORT  sPriority;
        PVOID  pvData;

DESCRIPTION

        Adds pvData (pointer) with sPriority to heap in psHeap.
      
RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      = element added
        C_NOTOK = element not added, heap full

NAME
        MemHepInit - initialize a heap before use

SYNOPSIS

        #include <memincs.h>

        SHORT APIENTRY
        MemHepInit(ppsHeap, sNumElms)
        HEAP_PP  ppsHeap;
        SHORT    sNumElms;

DESCRIPTION

        Initializes a heap through (*ppsHeap) to have sNumElms elements.
        
        The heap is implemented in MemHep*** functions by using a header
        structure (HEAP_T) which contains a pointer to an array of
        data structures (HEAP_DATA_T).   MemHepInit() must be called
        prior to any of the other MemHep*** functions.

        HEAP_P  psHeap;    /* main heap structure */
        PVOID   pvData;    /* for MemHepDeq() */
        SHORT   sPriority; /* for MemHepDeq() */

        C_STATUS = MemHepInit(&psHeap,10);  /* init for 10 members */
        
        C_STATUS = MemHepEnq(psHeap, 7, ptrToData);  /* prio = 7 */

        C_STATUS = MemHepDeq(psHeap, &sPriority, &pvData);


RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      - always returned


NAME
        MemHepSwap - swaps two HEAP_DATA_T elements

SYNOPSIS

        #include <memincs.h>

        SHORT APIENTRY
        MemHepSwap(psHeap1,psHeap2)
        HEAP_DATA_P  psHeap1;
        HEAP_DATA_P  psHeap2;

DESCRIPTION

        Uses memcpy() to swap two HEAP_DATA_T elements.
      
RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      - always returned


NAME
        MemHepUp - adjusts a heap after a Enque

SYNOPSIS

        #include <memincs.h>

        SHORT APIENTRY
        MemHepUp(psHeap)
        HEAP_P  psHeap;

DESCRIPTION

        Adjuts a heap through psHeap after a MemHepEnque() call.

        This is an MemHep*** internal function and there should
        not be any reason to call this from your program.
      
RESOURCE NAME

        MEMLIB.LIB

DIAGNOSTICS

        C_OK      - always returned



